function sdkp:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(sdkp, "sdkp", "sdkp", input)
    end
end

-- Purge handler
function sdkp:Purge(info)
    self.db.profile.snapshots = { };
    self.db.profile.loot = {};
    self:Print("Snapshot and loot data purged.");
    if self.BIDSCROLLFRAME then 
    	self.BIDSCROLLFRAME:SortData();
    end
end

-- Snapshot handler
function sdkp:CmdSnapshot(info, notes, loot)
	local snapshots = self.db.profile.snapshots;

	notes = strtrim(notes)
	if notes == "" then 
		notes = GetRealZoneText() .. " - Manual Snapshot";
	end
	
	local thissnap = snapshots[#snapshots];
	if not(thissnap) or thissnap.cols[1].value ~= notes then 
		-- take the snapshot
		table.insert(snapshots, {
			["cols"] = {
				{
					["value"] = notes,
				}, -- [1]
				{
					["value"] = time(),
				}, -- [2]
			},
			["onclickargs"] = {{ 
				-- double table because the outer table gets unpacked when passed to the onclick function
			}},
		});
		self:Debug("Snapshot inserted into snapshots table.");
		
		local nRaidMembers = GetNumRaidMembers();
		for raidIndex = 1, nRaidMembers do
			local raidMember = GetRaidRosterInfo(raidIndex);
			local main = self:GetAltInfo(raidMember);
			if main then 
				raidMember = main;	
			end
			table.insert(thissnap.onlcickargs[1], {
				["cols"] = {
					{
						["value"] = raidMember,
					}, -- [1]
				},
			});
		end
		
		self:SendGuild("["..notes.."] BOSS DOWN!!!");
		if loot then
			local count = 1;
			local message = ""; 
			for item in pairs(loot) do 
				message = message .. item;
				count = count + 1;
				if count == 3 then 
					self:SendGuild(message);
					message = "";
					count = 0;
				end
			end
			if message ~= "" then 
				self:SendGuild(message);
			end
		end
		self:SendGuild("ATTENDANCE CHECK! Response required from NON-RAIDED members within 3 minutes.");
		self:SendGuild("In order to get credit for this boss kill, you must [  /w " .. UnitName("player") .. " !here ]");
		self:ScheduleTimer(function() 
			self.checkingatt = false;
			self:SendGuild("ATTENDANCE CHECK HAS ENDED!  That is all...");
		end, 180, nil);
		self.checkingatt = true;
	end
	
	sdkp:UpdateSlideScrollData();
end

-- print debug mode handlers
function sdkp:IsDebug(info)
    return self.db.profile.debug;
end
function sdkp:ToggleDebug(info, value)
    self.db.profile.debug = value;
    SDKP_RAID_ZONE["Scarlet Monastery"] = self.db.profile.debug; -- easy to test on sm bosses, close to zone.
end

-- Loot tracking handlers
function sdkp:IsTrackLoot(info)
    return self.db.profile.trackLoot;
end
function sdkp:ToggleTrackLoot(info, value)
    self.db.profile.trackLoot = value;
end

-- Loot filter handlers
function sdkp:GetLootFilter(info, key)
    return self.db.profile.lootfilter[SDKP_COLORS[key]];
end
function sdkp:SetLootFilter(info, key, value)
	self.db.profile.lootfilter[SDKP_COLORS[key]] = value;
end

-- Ignore loot handlers
function sdkp:CmdIgnoreLoot(info, link)
	if link == "" then 
		self:Print("|cffffff78Usage:|r /sdkp ignoreloot [Item]    |cffffff78Example:|r /sdkp il |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r");
		self:Print(self:ListIgnoreLoot());
		return;
	end
	local _, _, item = string.find(link, "|c%x+|Hitem:%d+[:%d%-]+|h%[(.-)%]|h|r");
	if not item then 
		item = link;
	end		 
	if self.db.profile.ignoreloot[item] then
		self.db.profile.ignoreloot[item] = nil;
		self:Print("No longer ignoring " .. link);
	else
		self.db.profile.ignoreloot[item] = true;
		self:Print("Now ignoring " .. link);
	end
end
function sdkp:IsIgnoreLoot(item)
	return self.db.profile.ignoreloot[item] == true;
end
function sdkp:ListIgnoreLoot()
	local count = 0;
	for key, value in pairs(self.db.profile.ignoreloot) do
		count = count + 1;
	end
	return "Currently ignoring " .. count .. " items.";
end

-- Assign loot handlers
function sdkp:CmdAssignLoot(info, args)
	if args == "" then 
		self:Print("|cffffff78Usage:|r /sdkp assignloot [Item] player (reason/note)");
		self:Print("|cffffff78Example:|r /sdkp al |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r Sachmo");
		self:Print("|cffffff78Example:|r /sdkp al |cff0070dd|Hitem:32428:0:0:0:0:0:0:0|h[Heart of Darkness]|h|r Sachmo because he is awesome!");
		return;
	end
	string.gsub(args, "%s*(|c%x+|H[^|]+|h%[[^|]+%]|h|r)%s+([^%s]+)%s*(.*)", function(link, player, note)
		if not note or note == "" then 
			note = "Manually assigned";
		end
		self:CHAT_MSG_LOOT(note, player .. " receives loot: " .. link .. ".");
		return false;
	end);
end

-- Manual attendance/dkp snapshots handler
function sdkp:CmdHandleManualSnapshot(player, command, snapshot)
	if not player or strtrim(player) == "" then
		return;
	end
	player = strtrim(player);
	local main = self:GetAltInfo(player);
	if main then 
		player = main;
	end
	
	for i, row in ipairs(snapshot.onclickargs[1]) do 
		if row.cols[1].value == player and command == "add" then
			self:Print(player .. " was already in the snapshot.");
			return;
		elseif row.cols[1].value == player and command == "remove" then 
			table.remove(snapshot, i);
			self:Print(player .. " has been removed from the snapshot.");
			return;
		end
	end
	if command == "add" then 
		table.insert(snapshot.onclickargs[1], {
			["cols"] = {
				{
					["value"] = player,
				} -- [1]
			},
		});
		self:Print(player .. " added to the snapshot.");
	else 
		self:Print(player .. " was not in the snapshot.");
	end
	sdkp:UpdateSlideScrollData();
end

-- Manually assign major/minor noobery to a player
function sdkp:CmdNoobery(type, args)
	if not args or strtrim(args) == "" then
		return;
	end
	local _, _, player, note = args:find("([^%s]+)%s*(.*)");
	if not note or note == "" then
		note = "Just Because...";
	end
	local main = self:GetAltInfo(player);
	if main then player = main; end 
	
	local which, price = "Minor Noobery", 1;
	if type == "major" then 
		which, price = "Major Noobery", 2;
	end
	local message = player .. " receives loot: " .. which .. ". (" .. note .. ")";
	self:SendOfficer(message);
	self:SendRaid(message);
	
	table.insert(self.db.profile.loot, {
		["player"] = player, 
		["item"] = which, 
		["price"] = price, 
		["id"] = "", 
		["timestamp"] = time(), 
		["note"] = note,
		["color"] = "ffff8000",
	});
	if self.BIDSCROLLFRAME then 
		self.BIDSCROLLFRAME:SortData();
	end
end

-- Export handler
function sdkp:CmdExport(info)
	self:ExportAtt(info);
	self:ExportDkp(info);
end

-- Export Attendance
function sdkp:ExportAtt(info)
	local date = date("%m/%d/%y %H:%M:%S", self.db.profile.snapshot.timestamp);
	local exportdata = "<RaidInfo><key>"..date.."</key><start>"..date.."</start><end>"..date.."</end><zone></zone>";
	local count = 1;
	exportdata = exportdata.."<BossKills></BossKills><note>Attendance - "..self.db.profile.snapshot.notes.."</note><Join>";
	for player in pairs(self.db.profile.snapshot.attendanceList) do
		exportdata = exportdata.."<key"..count.."><player>"..player.."</player><time>"..date.."</time></key"..count..">";
		count = count + 1;
	end
	exportdata = exportdata.."</Join><Leave></Leave><Loot></Loot></RaidInfo>";
	
	self:EnqueueDialog(
		"CTRA Attendance Export",
		"Ok", 
		"Cancel", 
		function()end,
		function()end,
		exportdata
	); 
	self:ConfirmPopup();
end

-- Export Dkp 
function sdkp:ExportDkp(info)
	local ts = self.db.profile.snapshot.timestamp;
	local year = tonumber(date("%y", ts)) - 5;
	local time = date("%m/%d/"..year.." %H:%M:%S", ts);
	
	local exportdata = "<RaidInfo><key>"..time.."</key><start>"..time.."</start><end>"..time.."</end><zone></zone>";
	local count = 1;
	exportdata = exportdata.."<BossKills></BossKills><note>DKP - "..self.db.profile.snapshot.notes.."</note><Join>";
	for player in pairs(self.db.profile.snapshot.dkpList) do
		exportdata = exportdata.."<key"..count.."><player>"..player.."</player><time>"..time.."</time></key"..count..">";
		count = count + 1;
	end
	exportdata = exportdata.."</Join><Leave></Leave><Loot>";
	for count = 1, #self.db.profile.loot do
		local ldata = self.db.profile.loot[count];
		exportdata = exportdata.."<key"..count..">";
		exportdata = exportdata.."<ItemName>"..ldata.item.."</ItemName>";
		exportdata = exportdata.."<ItemID>"..ldata.id.."</ItemID>";
		exportdata = exportdata.."<Icon></Icon><Class></Class><SubClass></SubClass>";
		exportdata = exportdata.."<Color>"..ldata.color.."</Color>";
		exportdata = exportdata.."<Count>1</Count>";
		exportdata = exportdata.."<Player>"..ldata.player.."</Player>";
		exportdata = exportdata.."<Time>"..date("%m/%d/"..year.." %H:%M:%S", ldata.time).."</Time>";
		exportdata = exportdata.."<Zone></Zone><Boss></Boss>";
		exportdata = exportdata.."<Note>"..self.db.profile.snapshot.notes.."</Note>"
		exportdata = exportdata.."<Dkp>"..ldata.price.."</Dkp>"
		exportdata = exportdata.."</key"..count..">";
	end
	exportdata = exportdata.."</Loot></RaidInfo>";
	
	self:EnqueueDialog(
		"CTRA DKP Export",
		"Ok", 
		"Cancel", 
		function()end,
		function()end,
		exportdata
	); 
	self:ConfirmPopup();
end

-- sync handler
function sdkp:CmdSync(info, args)
	if not args or strtrim(args) == "" then
		self:Print("Who to sync with?");
		return;
	end
	local _, _, who = args:find("([^%s]+)");
	self.requestsync = who;
	self:SendCommMessage(self.name, self:Serialize("requestsync"), "WHISPER", who, "BULK");
end
